/****************************************************
  Projekt: TMCM-ARM (TMCL fr ARM-Module)

  Modul:   SPI-ARM.c
           Funktionen fr das SPI-Interface

  Datum:   23.3.2007 OK
*****************************************************/

#include "at91sam7x256.h"
#include "bits.h"
#include "Eval429.h"


/*******************************************************************
   Funktion: InitSPI()
   Parameter: ---

   Zweck: Initialisierung des SPI
********************************************************************/
void InitSPI(void)
{
  AT91C_BASE_PMC->PMC_PCER=1 << AT91C_ID_SPI0;
  AT91C_BASE_PMC->PMC_PCER=1 << AT91C_ID_SPI1;

  //SPI-Pins verbinden
  AT91C_BASE_PIOA->PIO_PDR=BIT24|BIT23|BIT22|BIT21|BIT18|BIT17|BIT16|BIT15|BIT14|BIT13|BIT12;
  AT91C_BASE_PIOA->PIO_ASR=BIT18|BIT17|BIT16|BIT15|BIT14|BIT13|BIT12;
  AT91C_BASE_PIOA->PIO_BSR=BIT24|BIT23|BIT22|BIT21;

  //Vorbereitung fr Sonderbehandlung des TMC424 (fnftes Gert an SPI0)
  AT91C_BASE_PIOA->PIO_PER=BIT25;
  AT91C_BASE_PIOA->PIO_OER=BIT25|BIT12;
  AT91C_BASE_PIOA->PIO_SODR=BIT25|BIT12;

  //SPI konfigurieren
  AT91C_BASE_SPI0->SPI_CR=AT91C_SPI_SWRST|AT91C_SPI_SPIEN;  //SPI-Reset
  AT91C_BASE_SPI0->SPI_CR=AT91C_SPI_SWRST|AT91C_SPI_SPIEN;  //SPI-Reset (zweimal wegen Rev. B Erratum)
  AT91C_BASE_SPI0->SPI_CR=AT91C_SPI_SPIEN;   //SPI ist eingeschaltet
  AT91C_BASE_SPI0->SPI_MR=AT91C_SPI_MODFDIS|AT91C_SPI_PS_VARIABLE|AT91C_SPI_MSTR;  //Master Mode, Chip Select ohne Dekoder
                                                                                  //MODFDIS ist hier sehr wichtig!!!
  AT91C_BASE_SPI0->SPI_CSR[0]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(24<<8);  //2MBit (Device 0: TMC429)
  AT91C_BASE_SPI0->SPI_CSR[1]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(24<<8);  //2MBit (Device 1: TMC262)
  AT91C_BASE_SPI0->SPI_CSR[2]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(24<<8);  //2MBit (Device 2: TMC261)
  AT91C_BASE_SPI0->SPI_CSR[3]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(24<<8);  //2MBit (Device 3: TMC260)

  AT91C_BASE_SPI1->SPI_CR=AT91C_SPI_SWRST|AT91C_SPI_SPIEN;  //SPI-Reset
  AT91C_BASE_SPI1->SPI_CR=AT91C_SPI_SWRST|AT91C_SPI_SPIEN;  //SPI-Reset (zweimal wegen Rev. B Erratum)
  AT91C_BASE_SPI1->SPI_CR=AT91C_SPI_SPIEN;   //SPI ist eingeschaltet
  AT91C_BASE_SPI1->SPI_MR=AT91C_SPI_MODFDIS|AT91C_SPI_PS_VARIABLE|AT91C_SPI_MSTR;  //Master Mode, Chip Select ohne Dekoder
                                                                                  //MODFDIS ist hier sehr wichtig!!!
  AT91C_BASE_SPI1->SPI_CSR[0]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(2<<8);   //24MBit (Device 0: ext. Flash, mit AMIC A25L016M-F bis zu 50MHz mglich)
  AT91C_BASE_SPI1->SPI_CSR[1]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(24<<8);  //2MBit (Device 1: nicht verwendet (TMC424 luft ber SPI0))
  AT91C_BASE_SPI1->SPI_CSR[2]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(24<<8);  //2MBit (Device 2: nicht verwendet)
  AT91C_BASE_SPI1->SPI_CSR[3]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(24<<8);  //2MBit (Device 3: nicht verwendet)
}


/*******************************************************************
   Funktion: ReadWriteSPI
   Parameter: DeviceNumber: Gertenummer (s. Konstanten in TMCM-ARM.h)
              Data: Zu sendendes Byte
              LastTransfer: FALSE: Gert bleibt nach dem Transfer selektiert (fr weitere Transfers)
                            TRUE: Gert wird nach dem Transfer deselektiert
   Zweck: Senden/Empfangen ber SPI
********************************************************************/
UCHAR ReadWriteSPI(UCHAR DeviceNumber, UCHAR Data, UCHAR LastTransfer)
{
  if((DeviceNumber & 0xf0)==0)
  {
    //Transfer initiieren
    if(LastTransfer)
      AT91C_BASE_SPI0->SPI_TDR=(DeviceNumber<<16)|Data|AT91C_SPI_LASTXFER;
    else
      AT91C_BASE_SPI0->SPI_TDR=(DeviceNumber<<16)|Data;


    //Warte bis Transfer beendet (Byte empfangen)
    while((AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_RDRF)==0);

    //Empfangenes Byte zurckgeben
    return AT91C_BASE_SPI0->SPI_RDR & 0xff;
  }
  else
  {
    if(DeviceNumber==SPI_DEV_TMC424)  //Sonderbehandlung fr TMC424 (Hardwarefehler auf Evalboard V1.10)
    {
      AT91C_BASE_PIOA->PIO_PER=BIT12;
      AT91C_BASE_PIOA->PIO_CODR=BIT25;
      AT91C_BASE_SPI0->SPI_TDR=Data|AT91C_SPI_LASTXFER;
      while((AT91C_BASE_SPI0->SPI_SR & AT91C_SPI_RDRF)==0);
      if(LastTransfer)
      {
        AT91C_BASE_PIOA->PIO_SODR=BIT25;
        AT91C_BASE_PIOA->PIO_PDR=BIT12;
      }

      return AT91C_BASE_SPI0->SPI_RDR & 0xff;
    }
    else
    {
      //Transfer initiieren
      if(LastTransfer)
        AT91C_BASE_SPI1->SPI_TDR=((DeviceNumber & 0x0f)<<16)|Data|AT91C_SPI_LASTXFER;
      else
        AT91C_BASE_SPI1->SPI_TDR=((DeviceNumber & 0x0f)<<16)|Data;

      //Warte bis Transfer beendet (Byte empfangen)
      while((AT91C_BASE_SPI1->SPI_SR & AT91C_SPI_RDRF)==0);

      //Empfangenes Byte zurckgeben
      return AT91C_BASE_SPI1->SPI_RDR & 0xff;
    }
  }
}


void SetSPISpeedTMC429(UCHAR SpeedDivider)
{
  AT91C_BASE_SPI0->SPI_CSR[0]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(SpeedDivider<<8);
}


void SetSPISpeedTMC26x(UCHAR SpeedDivider)
{
  AT91C_BASE_SPI0->SPI_CSR[1]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(SpeedDivider<<8);
  AT91C_BASE_SPI0->SPI_CSR[2]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(SpeedDivider<<8);
  AT91C_BASE_SPI0->SPI_CSR[3]=AT91C_SPI_CPOL|AT91C_SPI_CSAAT|(SpeedDivider<<8);
}
