/****************************************************
  Projekt: TMCM-ARM (TMCL fr ARM-Module)

  Modul:   TMCL-ARM.c
           TMCL-Interpreter

  Datum:   29.3.2007 OK
*****************************************************/


#include <stdlib.h>
#include "at91sam7x256.h"
#include "bits.h"

#include "Eval429.h"
#include "Eval429-UART.h"
#include "Eval429-Systimer.h"
#include "Eval429-TMCL.h"
#include "Eval429-TMC262.h"
#include "Eval429-SPI.h"
#include "Eval429-TMC428.h"
#include "Eval429-USB.h"
#include "Eval429-ADCIrq.h"
#include "Eval429-TMC424.h"
#include "Eval429-ExtFlash.h"
#include "Eval429-Special.h"

#define MODULE_ADDRESS 1
#define REPLY_ADDRESS 2

//Globale Variablen
static UCHAR TMCLCommandState;
TTMCLCommand ActualCommand;
TTMCLReply ActualReply;
UCHAR TMCLReplyFormat;
UCHAR SpecialReply[9];
static UCHAR UARTCmd[9];
static UCHAR UARTCount;
static UCHAR USBCmd[9];
static UCHAR USBReply[9];
static UCHAR ResetRequested;

static UINT VMax[N_O_MOTORS];
static UINT AMax[N_O_MOTORS];
static UCHAR VAModified[N_O_MOTORS];

extern UCHAR DriverFlags[N_O_MOTORS];
extern UINT StallLevel[N_O_MOTORS];
extern UCHAR SmartEnergy[N_O_MOTORS];


//Prototypen
static void RotateRight(void);
static void RotateLeft(void);
static void MotorStop(void);
static void MoveToPosition(void);
static void SetAxisParameter(void);
static void GetAxisParameter(void);
static void SetOutput(void);
static void GetInput(void);
static void GetVersion(void);
static void SoftwareReset(void);
static void UserFunc0(void);
static void UserFunc1(void);
static void UserFunc2(void);
static void UserFunc3(void);
static void UserFunc4(void);
static void UserFunc5(void);
static void UserFunc6(void);
static void UserFunc7(void);
static void Boot(void);

//Importierte Variablen
extern const char *VersionString;

//Funktionen

/*******************************************************************
   Funktion: ExecuteActualCommand()
   Parameter: ---

   Rckgabewert: ---

   Zweck: Ausfhren des Befehls, der in der globalen Variablen
          ActualCommand steht.
********************************************************************/
static void ExecuteActualCommand(void)
{
  //Antworttelegramm vorbelegen
  ActualReply.Opcode=ActualCommand.Opcode;
  ActualReply.Status=REPLY_OK;
  ActualReply.Value.Int32=ActualCommand.Value.Int32;

  //Befehl ausfhren
  switch(ActualCommand.Opcode)
  {
    case TMCL_ROR:
      RotateRight();
      break;

    case TMCL_ROL:
      RotateLeft();
      break;

    case TMCL_MST:
      MotorStop();
      break;

    case TMCL_MVP:
      MoveToPosition();
      break;

    case TMCL_SAP:
      SetAxisParameter();
      break;

    case TMCL_GAP:
      GetAxisParameter();
      break;

    case TMCL_SIO:
      SetOutput();
      break;

    case TMCL_GIO:
      GetInput();
      break;

    case TMCL_GetVersion:
      GetVersion();
      break;

    case TMCL_SoftwareReset:
      SoftwareReset();
      break;

    case TMCL_UF0:
      UserFunc0();
      break;

    case TMCL_UF1:
      UserFunc1();
      break;

    case TMCL_UF2:
      UserFunc2();
      break;

    case TMCL_UF3:
      UserFunc3();
      break;

    case TMCL_UF4:
      UserFunc4();
      break;

    case TMCL_UF5:
      UserFunc5();
      break;

    case TMCL_UF6:
      UserFunc6();
      break;

    case TMCL_UF7:
      UserFunc7();
      break;

    case TMCL_Boot:
      Boot();
      break;

    default:
      ActualReply.Status=REPLY_INVALID_CMD;
      break;
  }
}

/*******************************************************************
   Funktion: InitTMCL()
   Parameter: ---

   Rckgabewert: ---

   Zweck: Initialisierung der TMCL-Maschine
********************************************************************/
void InitTMCL(void)
{
  UINT i;

  RESET_WATCHDOG();

  for(i=0; i<N_O_MOTORS; i++)
  {
    VMax[i]=Read428Int((i<<5)|IDX_VMAX);
    AMax[i]=Read428Int((i<<5)|IDX_AMAX);
  }
}


/*******************************************************************
   Funktion: ProcessTMCL()
   Parameter: ---

   Rckgabewert: ---

   Zweck: Holen und Ausfhren von TMCL-Befehlen
          Diese Funktion mu periodisch aus der Hauptschleife
          heraus aufgerufen werden.
********************************************************************/
void ProcessTMCL(void)
{
  UCHAR Byte;
  UCHAR Checksum;
  UINT i;

  //**Antwort auf letzten Direktmodus-Befehl senden (wenn vorhanden)**
  if(TMCLReplyFormat==RF_NO_REPLY)
  {
    TMCLCommandState=TCS_IDLE;
    TMCLReplyFormat=RF_STANDARD;
  }

  if(TMCLCommandState==TCS_UART)  //Antwort ber UART
  {
    if(TMCLReplyFormat==RF_STANDARD)
    {
      Checksum=REPLY_ADDRESS+MODULE_ADDRESS+
               ActualReply.Status+ActualReply.Opcode+
               ActualReply.Value.Byte[3]+
               ActualReply.Value.Byte[2]+
               ActualReply.Value.Byte[1]+
               ActualReply.Value.Byte[0];

      WriteUART(REPLY_ADDRESS);
      WriteUART(MODULE_ADDRESS);
      WriteUART(ActualReply.Status);
      WriteUART(ActualReply.Opcode);
      WriteUART(ActualReply.Value.Byte[3]);
      WriteUART(ActualReply.Value.Byte[2]);
      WriteUART(ActualReply.Value.Byte[1]);
      WriteUART(ActualReply.Value.Byte[0]);
      WriteUART(Checksum);
    }
    else if(TMCLReplyFormat==RF_SPECIAL)
    {
      for(i=0; i<9; i++)
      {
        WriteUART(SpecialReply[i]);
      }
    }
  }
  else if(TMCLCommandState==TCS_UART_ERROR)  //letztes Kommando ber UART hatte falsche Prfsumme
  {
    ActualReply.Opcode=0;
    ActualReply.Status=REPLY_CHKERR;
    ActualReply.Value.Int32=0;

    Checksum=REPLY_ADDRESS+MODULE_ADDRESS+
             ActualReply.Status+ActualReply.Opcode+
             ActualReply.Value.Byte[3]+
             ActualReply.Value.Byte[2]+
             ActualReply.Value.Byte[1]+
             ActualReply.Value.Byte[0];

    WriteUART(REPLY_ADDRESS);
    WriteUART(MODULE_ADDRESS);
    WriteUART(ActualReply.Status);
    WriteUART(ActualReply.Opcode);
    WriteUART(ActualReply.Value.Byte[3]);
    WriteUART(ActualReply.Value.Byte[2]);
    WriteUART(ActualReply.Value.Byte[1]);
    WriteUART(ActualReply.Value.Byte[0]);
    WriteUART(Checksum);
  }
  else if(TMCLCommandState==TCS_USB)  //Antwort ber USB
  {
    //ShortDelay();
    if(TMCLReplyFormat==RF_STANDARD)
    {
      Checksum=REPLY_ADDRESS+MODULE_ADDRESS+
               ActualReply.Status+ActualReply.Opcode+
               ActualReply.Value.Byte[3]+
               ActualReply.Value.Byte[2]+
               ActualReply.Value.Byte[1]+
               ActualReply.Value.Byte[0];

      USBReply[0]=REPLY_ADDRESS;
      USBReply[1]=MODULE_ADDRESS;
      USBReply[2]=ActualReply.Status;
      USBReply[3]=ActualReply.Opcode;
      USBReply[4]=ActualReply.Value.Byte[3];
      USBReply[5]=ActualReply.Value.Byte[2];
      USBReply[6]=ActualReply.Value.Byte[1];
      USBReply[7]=ActualReply.Value.Byte[0];
      USBReply[8]=Checksum;
    }
    else if(TMCLReplyFormat==RF_SPECIAL)
    {
      for(i=0; i<9; i++)
      {
        USBReply[i]=SpecialReply[i];
      }
    }

    SendUSBReply(USBReply);
  }
  else if(TMCLCommandState==TCS_USB_ERROR)  //letztes Kommando ber USB hatte falsche Prfsumme
  {
    ActualReply.Opcode=0;
    ActualReply.Status=REPLY_CHKERR;
    ActualReply.Value.Int32=0;

    Checksum=REPLY_ADDRESS+MODULE_ADDRESS+
             ActualReply.Status+ActualReply.Opcode+
             ActualReply.Value.Byte[3]+
             ActualReply.Value.Byte[2]+
             ActualReply.Value.Byte[1]+
             ActualReply.Value.Byte[0];

    USBReply[0]=REPLY_ADDRESS;
    USBReply[1]=MODULE_ADDRESS;
    USBReply[2]=ActualReply.Status;
    USBReply[3]=ActualReply.Opcode;
    USBReply[4]=ActualReply.Value.Byte[3];
    USBReply[5]=ActualReply.Value.Byte[2];
    USBReply[6]=ActualReply.Value.Byte[1];
    USBReply[7]=ActualReply.Value.Byte[0];
    USBReply[8]=Checksum;

    SendUSBReply(USBReply);
  }

  //Zustand zurcksetzen (Antwort ist nun gesendet)
  TMCLCommandState=TCS_IDLE;
  TMCLReplyFormat=RF_STANDARD;

  //Letzter Befehl war Reset-Befehl => Reset
  if(ResetRequested) AT91C_BASE_RSTC->RSTC_RCR=AT91C_RSTC_PROCRST|AT91C_RSTC_PERRST|(0xA5<<24);

  if(ReadUART(&Byte))  //Befehl von RS232/RS485?
  {
    if(CheckUARTTimeout()) UARTCount=0;  //bei Timeout alles bisherige verwerfen
    UARTCmd[UARTCount++]=Byte;

    if(UARTCount==9)  //Neun Bytes wurden empfangen
    {
      UARTCount=0;

      if(UARTCmd[0]==MODULE_ADDRESS)  //Stimmt die Adresse?
      {
        Checksum=0;
        for(i=0; i<8; i++) Checksum+=UARTCmd[i];

        if(Checksum==UARTCmd[8])  //Stimmt die Prfsumme?
        {
          ActualCommand.Opcode=UARTCmd[1];
          ActualCommand.Type=UARTCmd[2];
          ActualCommand.Motor=UARTCmd[3];
          ActualCommand.Value.Byte[3]=UARTCmd[4];
          ActualCommand.Value.Byte[2]=UARTCmd[5];
          ActualCommand.Value.Byte[1]=UARTCmd[6];
          ActualCommand.Value.Byte[0]=UARTCmd[7];

          TMCLCommandState=TCS_UART;

          UARTCount=0;
        }
        else TMCLCommandState=TCS_UART_ERROR;  //Prfsumme war falsch
      }
    }
  }
  else if(GetUSBCmd(USBCmd))  //Befehl von USB?
  {
    if(USBCmd[0]==MODULE_ADDRESS)  //Stimmt die Adresse?
    {
      Checksum=0;
      for(i=0; i<8; i++) Checksum+=USBCmd[i];

      if(Checksum==USBCmd[8])  //Stimmt die Prfsumme?
      {
        ActualCommand.Opcode=USBCmd[1];
        ActualCommand.Type=USBCmd[2];
        ActualCommand.Motor=USBCmd[3];
        ActualCommand.Value.Byte[3]=USBCmd[4];
        ActualCommand.Value.Byte[2]=USBCmd[5];
        ActualCommand.Value.Byte[1]=USBCmd[6];
        ActualCommand.Value.Byte[0]=USBCmd[7];

        TMCLCommandState=TCS_USB;
      } else TMCLCommandState=TCS_USB_ERROR;  //Prfsumme war falsch
    }
  }


  //**Befehl ausfhren **
  //Befehl ausfhren, wenn erfolgreich ein Befehl geholt wurde.
  if(TMCLCommandState==TCS_UART || TMCLCommandState==TCS_USB) ExecuteActualCommand();
}


/*******************************************************************
   Funktion: CheckVAModified()
   Parameter: Motor: Motornummer

   Rckgabewert: ---

   Zweck: Prfen, ob VMAX oder AMAX gendert wurde und
          Wiederherstellen der durch den Benutzer eingestellten
          Werte.
********************************************************************/
static void CheckVAModified(UINT Motor)
{
  if(VAModified[Motor])
  {
    Write428Short(Motor<<5|IDX_VMAX, VMax[Motor]);
    SetAMax(Motor, AMax[Motor]);
    VAModified[Motor]=FALSE;
  }
}


//**************************** Funktionen fr die einzelnen TMCL-Befehle *****************
// Fr alle diese Funktionen gilt:
//   -die Parameter stehen in der globalen Variablen ActualCommand
//   -ein eventueller Rckgabewert wird in der globalen Variablen ActualReply abgelegt.
//****************************************************************************************

/******************************
   Funktion: RotateRight()

   Zweck: TMCL-Befehl ROR
 *******************************/
static void RotateRight(void)
{
  if(ActualCommand.Motor<N_O_MOTORS)
  {
    Set428RampMode(ActualCommand.Motor, RM_VELOCITY);
    Write428Short(ActualCommand.Motor<<5|IDX_VMAX, 2047);
    Write428Short(ActualCommand.Motor<<5|IDX_VTARGET, ActualCommand.Value.Int32);
  }
}


/******************************
   Funktion: RotateLeft()

   Zweck: TMCL-Befehl ROL
 *******************************/
static void RotateLeft(void)
{
  if(ActualCommand.Motor<N_O_MOTORS)
  {
    Set428RampMode(ActualCommand.Motor, RM_VELOCITY);
    Write428Short(ActualCommand.Motor<<5|IDX_VMAX, 2047);
    Write428Short(ActualCommand.Motor<<5|IDX_VTARGET, -ActualCommand.Value.Int32);
  }
}


/******************************
   Funktion: MotorStop()

   Zweck: TMCL-Befehl MST
 *******************************/
static void MotorStop(void)
{
  if(ActualCommand.Motor<N_O_MOTORS)
  {
    Set428RampMode(ActualCommand.Motor, RM_VELOCITY);
    Write428Short(ActualCommand.Motor<<5|IDX_VTARGET, 0);
  }
}


/******************************
   Funktion: MoveToPosition()

   Zweck: TMCL-Befehl MVP
 *******************************/
static void MoveToPosition(void)
{
  UINT NewPosition;

  if(ActualCommand.Motor>N_O_MOTORS)
  {
    ActualReply.Status=REPLY_INVALID_VALUE;
    return;
  }

  if((ActualCommand.Type==MVP_ABS || ActualCommand.Type==MVP_REL) && ActualCommand.Motor>=N_O_MOTORS)
  {
    ActualReply.Status=REPLY_INVALID_VALUE;
    return;
  }

  switch(ActualCommand.Type)
  {
    case MVP_ABS:
      CheckVAModified(ActualCommand.Motor);
      Write428Int(ActualCommand.Motor<<5|IDX_XTARGET, ActualCommand.Value.Int32);
      Set428RampMode(ActualCommand.Motor, RM_RAMP);
      break;

    case MVP_REL:
      CheckVAModified(ActualCommand.Motor);
      NewPosition=Read428Int(ActualCommand.Motor<<5|IDX_XACTUAL)+ActualCommand.Value.Int32;
      Write428Int(ActualCommand.Motor<<5|IDX_XTARGET, NewPosition);
      Set428RampMode(ActualCommand.Motor, RM_RAMP);
      ActualReply.Value.Int32=NewPosition;
      break;

    default:
      ActualReply.Status=REPLY_WRONG_TYPE;
      break;
  }
}


/******************************
   Funktion: SetAxisParameter

   Zweck: TMCL-Befehl SAP
 *******************************/
static void SetAxisParameter(void)
{
  UCHAR Read[4];

  if(ActualCommand.Motor>=N_O_MOTORS)
  {
    ActualReply.Status=REPLY_INVALID_VALUE;
    return;
  }

  switch(ActualCommand.Type)
  {
    case 0:
      Write428Int(ActualCommand.Motor<<5|IDX_XTARGET, ActualCommand.Value.Int32);
      break;

    case 1:
      Write428Int(ActualCommand.Motor<<5|IDX_XACTUAL, ActualCommand.Value.Int32);
      break;

    case 2:
      Write428Short(ActualCommand.Motor<<5|IDX_VTARGET, ActualCommand.Value.Int32);
      Write428Short(ActualCommand.Motor<<5|IDX_VMAX, abs(ActualCommand.Value.Int32));
      VAModified[ActualCommand.Motor]=TRUE;
      break;

    case 3:
      Write428Short(ActualCommand.Motor<<5|IDX_VACTUAL, ActualCommand.Value.Int32);
      break;

    case 4:
      Write428Short(ActualCommand.Motor<<5|IDX_VMAX, ActualCommand.Value.Int32);
      VMax[ActualCommand.Motor]=ActualCommand.Value.Int32;
      VAModified[ActualCommand.Motor]=FALSE;
      break;

    case 5:
      SetAMax(ActualCommand.Motor, ActualCommand.Value.Int32);
      AMax[ActualCommand.Motor]=ActualCommand.Value.Int32;
      VAModified[ActualCommand.Motor]=FALSE;
      break;

    case 6:
      Set262StallGuardCurrentScale(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 12:
      Read428Bytes(IDX_REFCONF_RM|ActualCommand.Motor<<5, Read);
      if(ActualCommand.Value.Int32!=0)
        Read[1]|=0x02;
      else
        Read[1]&= ~0x02;
      Write428Bytes(IDX_REFCONF_RM|ActualCommand.Motor<<5, Read);
      break;

    case 13:
      Read428Bytes(IDX_REFCONF_RM|ActualCommand.Motor<<5, Read);
      if(ActualCommand.Value.Int32!=0)
        Read[1]|=0x01;
      else
        Read[1]&= ~0x01;
      Write428Bytes(IDX_REFCONF_RM|ActualCommand.Motor<<5, Read);
      break;

    case 130:
      Write428Short(ActualCommand.Motor<<5|IDX_VMIN, ActualCommand.Value.Int32);
      break;

    case 135:
      Write428Short(ActualCommand.Motor<<5|IDX_AACTUAL, ActualCommand.Value.Int32);
      break;

    case 138:
      Read428Bytes(ActualCommand.Motor<<5|IDX_REFCONF_RM, Read);
      Read[2]=ActualCommand.Value.Byte[0];
      Write428Bytes(ActualCommand.Motor<<5|IDX_REFCONF_RM, Read);
      break;

    case 140:
      Set262StepDirMStepRes(ActualCommand.Motor, 8-ActualCommand.Value.Byte[0]);
      break;

    case 141:
      Write428Short(ActualCommand.Motor<<5|IDX_DX_REFTOLERANCE, ActualCommand.Value.Int32);
      break;

    case 149:
      Read428Bytes(IDX_REFCONF_RM|ActualCommand.Motor<<5, Read);
      if(ActualCommand.Value.Int32!=0)
        Read[1]|=0x04;
      else
        Read[1]&= ~0x04;
      Write428Bytes(IDX_REFCONF_RM|ActualCommand.Motor<<5, Read);
      break;

    case 153:
      Read428Bytes(ActualCommand.Motor<<5|IDX_PULSEDIV_RAMPDIV, Read);
      Read[1]=(Read[1] & 0xf0) | (ActualCommand.Value.Byte[0] & 0x0f);
      Write428Bytes(ActualCommand.Motor<<5|IDX_PULSEDIV_RAMPDIV, Read);
      break;

    case 154:
      Read428Bytes(ActualCommand.Motor<<5|IDX_PULSEDIV_RAMPDIV, Read);
      Read[1]=(Read[1] & 0x0f) | (ActualCommand.Value.Byte[0]<<4);
      Write428Bytes(ActualCommand.Motor<<5|IDX_PULSEDIV_RAMPDIV, Read);
      break;

    case 160:
      Set262StepDirInterpolation(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 161:
      Set262StepDirDoubleEdge(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 162:
      Set262ChopperBlankTime(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 163:
      Set262ChopperMode(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 164:
      Set262ChopperHysteresisDecay(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 165:
      Set262ChopperHysteresisEnd(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 166:
      Set262ChopperHysteresisStart(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 167:
      Set262ChopperTOff(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 168:
      Set262SmartEnergyIMin(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 169:
      Set262SmartEnergyDownStep(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 170:
      Set262SmartEnergyStallLevelMax(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 171:
      Set262SmartEnergyUpStep(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 172:
      Set262SmartEnergyStallLevelMin(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 173:
      Set262StallGuardFilter(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 174:
      Set262StallGuardThreshold(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 175:
      Set262DriverSlopeHighSide(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 176:
      Set262DriverSlopeLowSide(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 177:
      Set262DriverDisableProtection(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 178:
      Set262DriverProtectionTimer(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 179:
      Set262DriverVSenseScale(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 183:
      Set262DriverStepDirectionOff(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 184:
      Set262ChopperRandomTOff(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 185:
      Set262DriverTestMode(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 209:
      WriteEncoder(ActualCommand.Motor, ActualCommand.Value.Int32);
      break;

    case 210:
      SetEncoderPrescaler(ActualCommand.Motor, ActualCommand.Value.Int32>>5, ActualCommand.Value.Byte[0] & 0x1f);
      break;
      break;

    default:
      ActualReply.Status=REPLY_WRONG_TYPE;
      break;
  }
}


/******************************
   Funktion: GetAxisParameter()

   Zweck: TMCL-Befehl GAP
 *******************************/
static void GetAxisParameter(void)
{
  UCHAR Read[4];

  if(ActualCommand.Motor>=N_O_MOTORS)
  {
    ActualReply.Status=REPLY_INVALID_VALUE;
    return;
  }

  switch(ActualCommand.Type)
  {
    case 0:
      ActualReply.Value.Int32=Read428Int(ActualCommand.Motor<<5|IDX_XTARGET);
      break;

    case 1:
      ActualReply.Value.Int32=Read428Int(ActualCommand.Motor<<5|IDX_XACTUAL);
      break;

    case 2:
      ActualReply.Value.Int32=Read428Short(ActualCommand.Motor<<5|IDX_VTARGET);
      break;

    case 3:
      ActualReply.Value.Int32=Read428Short(ActualCommand.Motor<<5|IDX_VACTUAL);
      break;

    case 4:
      ActualReply.Value.Int32=VMax[ActualCommand.Motor];
      break;

    case 5:
      ActualReply.Value.Int32=AMax[ActualCommand.Motor];
      break;

    case 6:
      ActualReply.Value.Int32=Get262StallGuardCurrentScale(ActualCommand.Motor);
      break;

    case 8:
      ActualReply.Value.Int32=(Read428Status() & (1<<(ActualCommand.Motor*2))) ? 1:0;
      break;

    case 12:
      Read428Bytes(IDX_REFCONF_RM|ActualCommand.Motor<<5, Read);
      ActualReply.Value.Int32=(Read[1] & 0x02) ? 1:0;
      break;

    case 13:
      Read428Bytes(IDX_REFCONF_RM|ActualCommand.Motor<<5, Read);
      ActualReply.Value.Int32=(Read[1] & 0x01) ? 1:0;
      break;

    case 9:
    case 11:
      Read428Bytes(IDX_REF_SWITCHES, Read);  //linker Schalter
      ActualReply.Value.Int32=(Read[2] & (0x02<<ActualCommand.Motor*2)) ? 1:0;
      break;

    case 10:
      Read428Bytes(IDX_REF_SWITCHES, Read);  //rechter Schalter
      ActualReply.Value.Int32=(Read[2] & (0x01<<ActualCommand.Motor*2)) ? 1:0;
      break;

    case 130:
      ActualReply.Value.Int32=Read428Short(ActualCommand.Motor<<5|IDX_VMIN);
      break;

    case 135:
      ActualReply.Value.Int32=Read428Short(ActualCommand.Motor<<5|IDX_AACTUAL);
      break;

    case 138:
      Read428Bytes(ActualCommand.Motor<<5|IDX_REFCONF_RM, Read);
      ActualReply.Value.Int32=Read[2];
      break;

    case 140:
      ActualReply.Value.Int32=8-Get262StepDirMStepRes(ActualCommand.Motor);
      break;

    case 141:
      ActualReply.Value.Int32=Read428Short(ActualCommand.Motor<<5|IDX_DX_REFTOLERANCE);
      break;

    case 149:
      Read428Bytes(IDX_REFCONF_RM|ActualCommand.Motor<<5, Read);
      ActualReply.Value.Int32=(Read[1] & 0x04) ? 1:0;
      break;

    case 153:
      Read428Bytes(ActualCommand.Motor<<5|IDX_PULSEDIV_RAMPDIV, Read);
      ActualReply.Value.Int32=Read[1] & 0x0f;
      break;

    case 154:
      Read428Bytes(ActualCommand.Motor<<5|IDX_PULSEDIV_RAMPDIV, Read);
      ActualReply.Value.Int32=Read[1] >> 4;
      break;

    case 206:
      ActualReply.Value.Int32=StallLevel[ActualCommand.Motor];
      break;

    case 207:
      switch(ActualCommand.Motor)
      {
        case 0:
          ActualReply.Value.Int32=(AT91C_BASE_PIOB->PIO_PDSR & BIT9) ? 1:0;
          break;

        case 1:
          ActualReply.Value.Int32=(AT91C_BASE_PIOB->PIO_PDSR & BIT8) ? 1:0;
          break;

        case 2:
          ActualReply.Value.Int32=(AT91C_BASE_PIOB->PIO_PDSR & BIT14) ? 1:0;
          break;
      }
      break;

    case 208:
      ActualReply.Value.Int32=DriverFlags[ActualCommand.Motor];
      break;

    case 160:
      ActualReply.Value.Int32=Get262StepDirInterpolation(ActualCommand.Motor);
      break;

    case 161:
      ActualReply.Value.Int32=Get262StepDirDoubleEdge(ActualCommand.Motor);
      break;

    case 162:
      ActualReply.Value.Int32=Get262ChopperBlankTime(ActualCommand.Motor);
      break;

    case 163:
      ActualReply.Value.Int32=Get262ChopperMode(ActualCommand.Motor);
      break;

    case 164:
      ActualReply.Value.Int32=Get262ChopperHysteresisDecay(ActualCommand.Motor);
      break;

    case 165:
      ActualReply.Value.Int32=Get262ChopperHysteresisEnd(ActualCommand.Motor);
      break;

    case 166:
      ActualReply.Value.Int32=Get262ChopperHysteresisStart(ActualCommand.Motor);
      break;

    case 167:
      ActualReply.Value.Int32=Get262ChopperTOff(ActualCommand.Motor);
      break;

    case 168:
      ActualReply.Value.Int32=Get262SmartEnergyIMin(ActualCommand.Motor);
      break;

    case 169:
      ActualReply.Value.Int32=Get262SmartEnergyDownStep(ActualCommand.Motor);
      break;

    case 170:
      ActualReply.Value.Int32=Get262SmartEnergyStallLevelMax(ActualCommand.Motor);
      break;

    case 171:
      ActualReply.Value.Int32=Get262SmartEnergyUpStep(ActualCommand.Motor);
      break;

    case 172:
      ActualReply.Value.Int32=Get262SmartEnergyStallLevelMin(ActualCommand.Motor);
      break;

    case 173:
      ActualReply.Value.Int32=Get262StallGuardFilter(ActualCommand.Motor);
      break;

    case 174:
      ActualReply.Value.Int32=Get262StallGuardThreshold(ActualCommand.Motor);
      break;

    case 175:
      ActualReply.Value.Int32=Get262DriverSlopeHighSide(ActualCommand.Motor);
      break;

    case 176:
      ActualReply.Value.Int32=Get262DriverSlopeLowSide(ActualCommand.Motor);
      break;

    case 177:
      ActualReply.Value.Int32=Get262DriverDisableProtection(ActualCommand.Motor);
      break;

    case 178:
      ActualReply.Value.Int32=Get262DriverProtectionTimer(ActualCommand.Motor);
      break;

    case 179:
      ActualReply.Value.Int32=Get262DriverVSenseScale(ActualCommand.Motor);
      break;

    case 180:
      ActualReply.Value.Int32=SmartEnergy[ActualCommand.Motor];
      break;

    case 182:
      if(IS_SD_MODE_SELECTED() || ActualCommand.Motor!=0)
      {
        if(Get262DriverReadSelect(ActualCommand.Motor)!=TMC262_RB_MSTEP)
          Set262DriverReadSelect(ActualCommand.Motor, TMC262_RB_MSTEP);
        ActualReply.Value.Int32=0;
        Read262State(ActualCommand.Motor, &ActualReply.Value.Byte[1], &ActualReply.Value.Byte[0], NULL, NULL, NULL);
      }
      else
      {
        ActualReply.Value.Byte[1]=0x00;
        ActualReply.Value.Byte[0]=0x00;
      }
      break;

    case 183:
      ActualReply.Value.Int32=Get262DriverStepDirectionOff(ActualCommand.Motor);
      break;

    case 184:
      ActualReply.Value.Int32=Get262ChopperRandomTOff(ActualCommand.Motor);
      break;

    case 209:
      ActualReply.Value.Int32=ReadEncoder(ActualCommand.Motor);
      break;

    default:
      ActualReply.Status=REPLY_WRONG_TYPE;
      break;
  }
}


/******************************
   Funktion: SetOutput()

   Zweck: TMCL-Befehl SIO
 *******************************/
static void SetOutput(void)
{
  if(ActualCommand.Motor==2)
  {
    switch(ActualCommand.Type)
    {
      case 0:
        if(ActualCommand.Value.Int32==0)
          AT91C_BASE_PIOB->PIO_CODR=BIT20;
        else
          AT91C_BASE_PIOB->PIO_SODR=BIT20;
        break;

      case 1:
        if(ActualCommand.Value.Int32==0)
          AT91C_BASE_PIOB->PIO_CODR=BIT24;
        else
          AT91C_BASE_PIOB->PIO_SODR=BIT24;
        break;

      case 2:
        if(ActualCommand.Value.Int32==0)
          AT91C_BASE_PIOB->PIO_CODR=BIT25;
        else
          AT91C_BASE_PIOB->PIO_SODR=BIT25;
        break;

      default:
        ActualReply.Status=REPLY_WRONG_TYPE;
        break;
    }
  }
  else ActualReply.Status=REPLY_INVALID_VALUE;
}


/******************************
   Funktion: GetInput()

   Zweck: TMCL-Befehl GIO
 *******************************/
static void GetInput(void)
{
  switch(ActualCommand.Motor)
  {
    case 0:
      if(ActualCommand.Type<8)
        ActualReply.Value.Int32=GetADCValue(ActualCommand.Type)>300 ? 1:0;
      else
        ActualReply.Status=REPLY_WRONG_TYPE;
      break;

    case 1:
      if(ActualCommand.Type<8)
        ActualReply.Value.Int32=GetADCValue(ActualCommand.Type);
      else
        ActualReply.Status=REPLY_WRONG_TYPE;
      break;

    case 2:
      switch(ActualCommand.Type)
      {
        case 0:
          ActualReply.Value.Int32=(AT91C_BASE_PIOB->PIO_ODSR & BIT20) ? 1:0;
          break;

        case 1:
          ActualReply.Value.Int32=(AT91C_BASE_PIOB->PIO_ODSR & BIT24) ? 1:0;
          break;

        case 2:
          ActualReply.Value.Int32=(AT91C_BASE_PIOB->PIO_ODSR & BIT25) ? 1:0;
          break;

        default:
          ActualReply.Status=REPLY_WRONG_TYPE;
          break;
      }
      break;

    default:
      ActualReply.Status=REPLY_INVALID_VALUE;
      break;
  }
}


/*********************************
   Funktion: GetVersion()

   Zweck: TMCL-Befehl 136
          (get version)
 *********************************/
static void GetVersion(void)
{
  UINT i;

  if(ActualCommand.Type==0)
  {
    TMCLReplyFormat=RF_SPECIAL;
    SpecialReply[0]=MODULE_ADDRESS;
    for(i=0; i<8; i++)
      SpecialReply[i+1]=VersionString[i];
  }
  else if(ActualCommand.Type==1)
  {
    ActualReply.Value.Byte[3]=SW_TYPE_HIGH;
    ActualReply.Value.Byte[2]=SW_TYPE_LOW;
    ActualReply.Value.Byte[1]=SW_VERSION_HIGH;
    ActualReply.Value.Byte[0]=SW_VERSION_LOW;
  }
}


/*********************************
   Funktion: SoftwareReset()

   Zweck: TMCL-Befehl 255
          (software reset)
 *********************************/
static void SoftwareReset(void)
{
  if(ActualCommand.Value.Int32==1234) ResetRequested=TRUE;
}


/*********************************
   Funktion: UserFunc0()

   Zweck: TMCL-Befehl UF0
   Hier verwendet zum Schalten der
   ENABLE-Leitungen.
 *********************************/
static void UserFunc0(void)
{
  if(ActualCommand.Value.Int32==0)
    switch(ActualCommand.Motor)
    {
      case 0:
        DISABLE_DRIVER1();
        break;

      case 1:
        DISABLE_DRIVER2();
        break;

      case 2:
        DISABLE_DRIVER3();
        break;

      default:
        ActualReply.Status=REPLY_INVALID_VALUE;
        break;
    }
  else
    switch(ActualCommand.Motor)
    {
      case 0:
        ENABLE_DRIVER1();
        break;

      case 1:
        ENABLE_DRIVER2();
        break;

      case 2:
        ENABLE_DRIVER3();
        break;

      default:
        ActualReply.Status=REPLY_INVALID_VALUE;
        break;
    }
}


/*********************************
   Funktion: UserFunc1()

   Zweck: TMCL-Befehl UF1
   Hier verwendet zum Setzen der
   Taktfrequenz.
 *********************************/
static void UserFunc1(void)
{
  switch(ActualCommand.Type)
  {
    case 0:
      switch(ActualCommand.Value.Int32)
      {
        case 0:
          AT91C_BASE_PMC->PMC_PCKR[2]=AT91C_PMC_PRES_CLK|AT91C_PMC_CSS_MAIN_CLK;   //16Mhz an PCK2 (PB22)
          SetSPISpeedTMC26x(24);
          break;

        case 1:
          AT91C_BASE_PMC->PMC_PCKR[2]=AT91C_PMC_PRES_CLK_2|AT91C_PMC_CSS_MAIN_CLK;   //8Mhz an PCK2 (PB22)
          SetSPISpeedTMC26x(24);
          break;

        case 2:
          AT91C_BASE_PMC->PMC_PCKR[2]=AT91C_PMC_PRES_CLK_4|AT91C_PMC_CSS_MAIN_CLK;   //4Mhz an PCK2 (PB22)
          SetSPISpeedTMC26x(24);
          break;

        case 3:
          AT91C_BASE_PMC->PMC_PCKR[2]=AT91C_PMC_PRES_CLK_8|AT91C_PMC_CSS_MAIN_CLK;   //2Mhz an PCK2 (PB22)
          SetSPISpeedTMC26x(48);
          break;

        case 4:
          AT91C_BASE_PMC->PMC_PCKR[2]=AT91C_PMC_PRES_CLK_16|AT91C_PMC_CSS_MAIN_CLK;   //1Mhz an PCK2 (PB22)
          SetSPISpeedTMC26x(96);
          break;

        case 5:
          AT91C_BASE_PMC->PMC_PCKR[2]=AT91C_PMC_PRES_CLK_32|AT91C_PMC_CSS_MAIN_CLK;   //500kHz an PCK2 (PB22)
          SetSPISpeedTMC26x(192);
          break;

        case 6:
          AT91C_BASE_PMC->PMC_PCKR[2]=AT91C_PMC_PRES_CLK_64|AT91C_PMC_CSS_MAIN_CLK;   //250kHz an PCK2 (PB22)
          SetSPISpeedTMC26x(255);
          break;

        default:
          ActualReply.Status=REPLY_INVALID_VALUE;
          break;
      }
      break;

    case 1:
      switch(ActualCommand.Value.Int32)
      {
        case 0:
          AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK|AT91C_PMC_CSS_MAIN_CLK;   //16Mhz an PCK1 (PB21)
          SetSPISpeedTMC429(24);
          break;

        case 1:
          AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK_2|AT91C_PMC_CSS_MAIN_CLK;   //8Mhz an PCK1 (PB21)
          SetSPISpeedTMC429(48);
          break;

        case 2:
          AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK_4|AT91C_PMC_CSS_MAIN_CLK;   //4Mhz an PCK1 (PB21)
          SetSPISpeedTMC429(96);
          break;

        case 3:
          AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK_8|AT91C_PMC_CSS_MAIN_CLK;   //2Mhz an PCK1 (PB21)
          SetSPISpeedTMC429(192);
          break;

        case 4:
          AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK_16|AT91C_PMC_CSS_MAIN_CLK;   //1Mhz an PCK1 (PB21)
          SetSPISpeedTMC429(255);
          break;

        case 5:
          AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK_32|AT91C_PMC_CSS_MAIN_CLK;   //500kHz an PCK1 (PB21)
          SetSPISpeedTMC429(255);
          break;

        case 6:
          AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK_64|AT91C_PMC_CSS_MAIN_CLK;   //250kHz an PCK1 (PB21)
          SetSPISpeedTMC429(255);
          break;

        case 7:
          AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK_4|AT91C_PMC_CSS_PLL_CLK;   //24Mhz an PCK1 (PB21)
          SetSPISpeedTMC429(24);
          break;

        case 8:
          AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK_2|AT91C_PMC_CSS_PLL_CLK;   //48Mhz an PCK1 (PB21)
          SetSPISpeedTMC429(24);
          break;

        default:
          ActualReply.Status=REPLY_INVALID_VALUE;
          break;
      }
      break;

    default:
      ActualReply.Status=REPLY_WRONG_TYPE;
      break;
  }
}


/*********************************
   Funktion: UserFunc2()

   Zweck: TMCL-Befehl UF2
   Hier verwendet zum Umschalten
   zwischen S/D-und SPI-Betrieb.
 *********************************/
static void UserFunc2(void)
{
  switch(ActualCommand.Type)
  {
    case 0:
      switch(ActualCommand.Value.Int32)
      {
        case 0:
          SelectStepDirMode();
          break;

        case 1:
          SelectSPIMode();
          break;

        default:
          ActualReply.Status=REPLY_INVALID_VALUE;
          break;
      }
      break;

    case 1:
      if(IS_SD_MODE_SELECTED())
        ActualReply.Value.Int32=0;
      else
        ActualReply.Value.Int32=1;
      break;

    default:
      ActualReply.Status=REPLY_WRONG_TYPE;
      break;
  }
}


static void UserFunc3(void)
{
}


static void UserFunc4(void)
{
}


static void UserFunc5(void)
{
}


static void UserFunc6(void)
{
}


/*********************************
   Funktion: UserFunc7()

   Zweck: TMCL-Befehl UF7
   Hier verwendet fr direkten
   SPI-Zugriff auf die Chips.
 *********************************/
static void UserFunc7(void)
{
  switch(ActualCommand.Type)
  {
    case 0:  //TMC429
      ActualReply.Value.Byte[3]=ReadWriteSPI(SPI_DEV_TMC429, ActualCommand.Value.Byte[3], FALSE);
      ActualReply.Value.Byte[2]=ReadWriteSPI(SPI_DEV_TMC429, ActualCommand.Value.Byte[2], FALSE);
      ActualReply.Value.Byte[1]=ReadWriteSPI(SPI_DEV_TMC429, ActualCommand.Value.Byte[1], FALSE);
      ActualReply.Value.Byte[0]=ReadWriteSPI(SPI_DEV_TMC429, ActualCommand.Value.Byte[0], TRUE);
      break;

    case 1:  //TMC262
      ActualReply.Value.Byte[2]=ReadWriteSPI(SPI_DEV_TMC262, ActualCommand.Value.Byte[2], FALSE);
      ActualReply.Value.Byte[1]=ReadWriteSPI(SPI_DEV_TMC262, ActualCommand.Value.Byte[1], FALSE);
      ActualReply.Value.Byte[0]=ReadWriteSPI(SPI_DEV_TMC262, ActualCommand.Value.Byte[0], TRUE);
      break;

    case 2:  //TMC261
      ActualReply.Value.Byte[2]=ReadWriteSPI(SPI_DEV_TMC261, ActualCommand.Value.Byte[2], FALSE);
      ActualReply.Value.Byte[1]=ReadWriteSPI(SPI_DEV_TMC261, ActualCommand.Value.Byte[1], FALSE);
      ActualReply.Value.Byte[0]=ReadWriteSPI(SPI_DEV_TMC261, ActualCommand.Value.Byte[0], TRUE);
      break;

    case 3:  //TMC260
      ActualReply.Value.Byte[2]=ReadWriteSPI(SPI_DEV_TMC260, ActualCommand.Value.Byte[2], FALSE);
      ActualReply.Value.Byte[1]=ReadWriteSPI(SPI_DEV_TMC260, ActualCommand.Value.Byte[1], FALSE);
      ActualReply.Value.Byte[0]=ReadWriteSPI(SPI_DEV_TMC260, ActualCommand.Value.Byte[0], TRUE);
      break;

    case 4:  //TMCM424
      ActualReply.Value.Byte[3]=ReadWriteSPI(SPI_DEV_TMC424, ActualCommand.Value.Byte[3], FALSE);
      ActualReply.Value.Byte[2]=ReadWriteSPI(SPI_DEV_TMC424, ActualCommand.Value.Byte[2], FALSE);
      ActualReply.Value.Byte[1]=ReadWriteSPI(SPI_DEV_TMC424, ActualCommand.Value.Byte[1], FALSE);
      ActualReply.Value.Byte[0]=ReadWriteSPI(SPI_DEV_TMC424, ActualCommand.Value.Byte[0], TRUE);
      break;

    default:
      ActualReply.Status=REPLY_WRONG_TYPE;
      break;
  }
}


/****************************************
   Funktion: Boot()

   Zweck: Spezialbefehl zum Aufruf des
          Bootloaders.
 ***************************************/
static void Boot(void)
{
  if(ActualCommand.Type==0x81 && ActualCommand.Motor==0x92 &&
     ActualCommand.Value.Byte[3]==0xa3 && ActualCommand.Value.Byte[2]==0xb4 &&
     ActualCommand.Value.Byte[1]==0xc5 && ActualCommand.Value.Byte[0]==0xd6)
  {
    if(TMCLCommandState==TCS_USB)
    {
      DISABLE_DRIVERS();
      AT91F_DisableInterrupts();
      AT91C_BASE_PIOA->PIO_SODR=BIT6;
      DeleteMagicNumber();
      AT91C_BASE_RSTC->RSTC_RCR=AT91C_RSTC_PROCRST|AT91C_RSTC_PERRST|(0xA5<<24);
    }

    DISABLE_DRIVERS();
    AT91F_DisableInterrupts();
    AT91C_BASE_AIC->AIC_IDCR = 0xFFFFFFFF;
    AT91C_BASE_AIC->AIC_ICCR = 0xFFFFFFFF;

    AT91C_BASE_RSTC->RSTC_RCR=AT91C_RSTC_PROCRST|(0xA5<<24);
  }
}
