/****************************************************
  Projekt: TMCM-ARM

  Modul:   UARTIrq-ARM.c
           UART Interrupt Handler

  Datum:   23.3.2007 OK
*****************************************************/

#include "at91sam7x256.h"
#include "bits.h"
#include "Eval429.h"
#include "Eval429-UART.h"


//Puffer fr Senden und Empfangen
volatile UCHAR UARTRxBuffer[UART_BUFFER_SIZE];
volatile UCHAR UARTTxBuffer[UART_BUFFER_SIZE];
volatile UINT UARTRxReadPtr, UARTRxWritePtr, UARTTxReadPtr, UARTTxWritePtr;
volatile UCHAR UARTTimeoutFlag;
volatile UINT UARTTxDelay;
volatile UINT UARTTxDelayTimer;


/*******************************************************************
  UART-Interrupthandler
  Wird durch den AIC aufgerufen, wenn ein UART-Interrupt auftritt.
  Dies paasiert, wenn ein Zeichen angekommen ist oder die
  ein Zeichen gesendet werden kann.
********************************************************************/
void UARTInterruptHandler(void)
{
  UINT CSR;
  UINT i;
  UCHAR Byte;

  //Status der UART merken (dies setzt auch den Interrupt zurck)
  CSR=UART0->US_CSR;

  //Ist ein Zeichen angekommen?
  if(CSR & AT91C_US_RXRDY)
  {
    //Sendeverzgerung neu starten
    UARTTxDelayTimer=UARTTxDelay;

    //Zeichen in jedem Fall abholen (auch wenn dieses nicht kopiert werden kann)
    Byte=UART0->US_RHR;

    //Zeichen in den Empfangspuffer kopieren (wenn dort Platz ist)
    i=UARTRxWritePtr+1;
    if(i==UART_BUFFER_SIZE) i=0;

    if(i!=UARTRxReadPtr)
    {
      UARTRxBuffer[UARTRxWritePtr]=Byte;
      UARTRxWritePtr=i;
    }
  }

  //Kann das nchste Zeichen gesendet werden (Senderegister frei und Verzgerung abgelaufen)?
  if((CSR & AT91C_US_TXRDY) && UARTTxDelayTimer==0)
  {
    if(UARTTxWritePtr!=UARTTxReadPtr)
    {
      UART0->US_THR=UARTTxBuffer[UARTTxReadPtr++];
      if(UARTTxReadPtr==UART_BUFFER_SIZE) UARTTxReadPtr=0;
    }
    else
    {
      //Sendeinterrupt deaktivieren, wenn kein Zeichen im Sendepuffer ist
      UART0->US_IDR=AT91C_US_TXRDY;
    }
  }

  //Ist ein Timeout aufgetreten?
  if(CSR & AT91C_US_TIMEOUT)
  {
    UARTTimeoutFlag=TRUE;
    UART0->US_CR=AT91C_US_STTTO;  //Receiver-Timeout zurcksetzen, beginnt automatisch wieder wenn das nchste Byte kommt.
  }
}


