/****************************************************
  Projekt: TMCM-ARM  (TMCL fr ARM-Module)

  Modul:   TMCM-ARM.h
           Allgemeine Definitionen fr das TMCM-ARM.

  Datum:   29.1.2007 OK
*****************************************************/

#ifndef __EVAL429_H
#define __EVAL429_H

//Allgemeine Definitionen
typedef unsigned char UCHAR;
typedef unsigned short USHORT;
typedef unsigned int UINT;

#define FASTRUN __attribute__ ((long_call, section (".fastrun")))

#define TRUE 1
#define FALSE 0

#define MIN(a,b) (a<b) ? (a) : (b)
#define MAX(a,b) (a>b) ? (a) : (b)

#define SW_TYPE_LOW 0xAD
#define SW_TYPE_HIGH 0x01
#define SW_VERSION_LOW 4
#define SW_VERSION_HIGH 2

//Allgemeine Definitionen
#define TIMER_INTERRUPT_PRIORITY 7
#define UART_INTERRUPT_PRIORITY 6
#define CAN_INTERRUPT_PRIORITY 5
#define ADC_INTERRUPT_PRIORITY 0

#define RESET_WATCHDOG() AT91C_BASE_WDTC->WDTC_WDCR=(0xA5 << 24)|AT91C_WDTC_WDRSTT

#define SPI_DEV_TMC429 0x00
#define SPI_DEV_TMC262 0x01
#define SPI_DEV_TMC261 0x03
#define SPI_DEV_TMC260 0x07
#define SPI_DEV_FLASH  0x10
#define SPI_DEV_TMC424 0x11

#define N_O_MOTORS 3

#define DISABLE_DRIVERS() AT91C_BASE_PIOA->PIO_SODR=(BIT10|BIT9|BIT8)
#define ENABLE_DRIVERS() AT91C_BASE_PIOA->PIO_CODR=(BIT10|BIT9|BIT8)

#define DISABLE_DRIVER1() AT91C_BASE_PIOA->PIO_SODR=BIT8
#define DISABLE_DRIVER2() AT91C_BASE_PIOA->PIO_SODR=BIT9
#define DISABLE_DRIVER3() AT91C_BASE_PIOA->PIO_SODR=BIT10
#define ENABLE_DRIVER1() AT91C_BASE_PIOA->PIO_CODR=BIT8
#define ENABLE_DRIVER2() AT91C_BASE_PIOA->PIO_CODR=BIT9
#define ENABLE_DRIVER3() AT91C_BASE_PIOA->PIO_CODR=BIT10

#define LED_LEFT_ON()   (AT91C_BASE_PIOB->PIO_CODR=BIT3)
#define LED_LEFT_OFF()  (AT91C_BASE_PIOB->PIO_SODR=BIT3)
#define LED_RIGHT_ON()  (AT91C_BASE_PIOB->PIO_CODR=BIT4)
#define LED_RIGHT_OFF() (AT91C_BASE_PIOB->PIO_SODR=BIT4)
#define LED_COOL_ON()   (AT91C_BASE_PIOB->PIO_SODR=BIT5)
#define LED_COOL_OFF()  (AT91C_BASE_PIOB->PIO_CODR=BIT5)

#define DRIVE_MODE_SPI()      AT91C_BASE_PIOA->PIO_CODR=BIT11
#define DRIVE_MODE_SD()       AT91C_BASE_PIOA->PIO_SODR=BIT11
#define IS_SD_MODE_SELECTED() (AT91C_BASE_PIOA->PIO_ODSR & BIT11)

#define BUTTON_LEFT() !(AT91C_BASE_PIOB->PIO_PDSR & BIT7)
#define BUTTON_RIGHT() !(AT91C_BASE_PIOB->PIO_PDSR & BIT1)
#define BUTTON_COOL() !(AT91C_BASE_PIOB->PIO_PDSR & BIT2)
#define BUTTON_DEBOUNCE_MAX 100

#define MAX_LOG_ENTRY 104857

typedef struct
{
	UINT Timestamp;
	int XTarget;
	int XActual;
	short VTarget;
	short VActual;
	short AActual;
	short StallGuard;
} TLogEntry;


//Allgemeine Funktionen
extern void AT91F_DisableInterrupts(void);
extern void AT91F_EnableInterrupts(void);

#endif
